/*
    Lex
 */

extern int yylineno; /* from lexer */
extern FILE *yyin;

/*
    Expression
*/

typedef enum Boolean {
    B_FALSE = 0,
    B_TRUE = 1
} Boolean;

typedef struct Expression_tag Expression;

typedef enum {
    INT_EXPRESSION = 1,
    DOUBLE_EXPRESSION,
 
    STRING_EXPRESSION,
    MINUS_EXPRESSION,
    
    ADD_EXPRESSION,     /* 5 */
    SUB_EXPRESSION,
    MUL_EXPRESSION,
    DIV_EXPRESSION,
    MOD_EXPRESSION,
    
    AND_EXPRESSION,      /* 10 */
    OR_EXPRESSION,
    XOR_EXPRESSION,
    NOT_EXPRESSION,

    EQ_EXPRESSION,          /* 13 */
    GT_EXPRESSION,     
    LT_EXPRESSION,
    NE_EXPRESSION,     
    GE_EXPRESSION,     
    LE_EXPRESSION,

    ARRAY_EXPRESSION,

    COMMA_EXPRESSION,     /* 20 */
    SEMICOLON_EXPRESSION,
    EMPTY_EXPRESSION,
    EMPTY_START_EXPRESSION,
    
    ASC_EXPRESSION,
    CHR_EXPRESSION,
    VAL_EXPRESSION,
    STR_EXPRESSION,
    MID_EXPRESSION,
    HEX_EXPRESSION,
    LEN_EXPRESSION,
    
    RND_EXPRESSION,
    ABS_EXPRESSION,
    SGN_EXPRESSION,
    RAD_EXPRESSION,
    DEG_EXPRESSION,
    SIN_EXPRESSION,
    COS_EXPRESSION,
    TAN_EXPRESSION,
    ATAN_EXPRESSION,
    ATAN_YX_EXPRESSION,
    SQR_EXPRESSION,
    EXP_EXPRESSION,
    LOG_EXPRESSION,
    FLOOR_EXPRESSION,

    LABEL_EXPRESSION,
    
    IDENTIFIER_EXPRESSION
} ExpressionType;

typedef struct {
    Expression  *left;
    Expression  *right;
} BinaryExpression;

typedef struct {
    Expression  *source;
    Expression  *location;
    Expression  *size;
} MidExpression;

typedef enum {
    IDENTIFIER_VALUE_INT = 1,
    IDENTIFIER_VALUE_DOUBLE,
    IDENTIFIER_VALUE_STRING,
    IDENTIFIER_VALUE_NUMBER_ARRAY,
    IDENTIFIER_VALUE_STRING_ARRAY
} IdentifierValueType;

typedef enum {
    IDENTIFIER_DIMENSION0 = 1,
    IDENTIFIER_DIMENSION1,
    IDENTIFIER_DIMENSION2
} IdentifierDimensionType;

typedef struct {
    char *identifier;
    IdentifierValueType valueType;
    IdentifierDimensionType dimensionType;
    Expression *dimension1;
    Expression *dimension2;    
} IdentifierType;

typedef struct ExpressionList_tag {
    Expression *expression;
    struct ExpressionList_tag *next;
} ExpressionList;

struct Expression_tag {
    ExpressionType type;
    int line_number;
    union {
        int int_value;
        double double_value;
        char *string_value;
        char *label;
        Expression *expression;
        BinaryExpression binary_expression;
        MidExpression mid_expression;
        Expression *bit_not;
        ExpressionList *array_literal;
        IdentifierType identifier;
    } u;
};

Expression *create_alloc_expression();
Expression *create_minus_expression(Expression *exp);
Expression *create_binary_expression(ExpressionType type, Expression *left, Expression *right);
Expression *create_not_expression(Expression *exp);
Expression *create_identifier_expression(char *name, IdentifierValueType valueType, IdentifierDimensionType dimensionType,  Expression *dimemsion1,  Expression *dimension2);
ExpressionList *create_alloc_expression_List();
ExpressionList *create_array_expression(Expression *exp);
ExpressionList *chain_array_expression(ExpressionList *list, Expression *exp);
Expression *create_asc_expression(Expression *exp);
Expression *create_chr_expression(Expression *exp);
Expression *create_val_expression(Expression *exp);
Expression *create_str_expression(Expression *exp);
Expression *create_mid_expression(Expression *source, Expression *location, Expression *size);
Expression *create_hex_expression(Expression *exp);
Expression *create_len_expression(Expression *exp);
Expression *create_rnd_expression(Expression *exp);
Expression *create_lable_expression(char *label);
Expression *create_abs_expression(Expression *exp);
Expression *create_sgn_expression(Expression *exp);
Expression *create_pi_expression();
Expression *create_rad_expression(Expression *exp);
Expression *create_deg_expression(Expression *exp);
Expression *create_sin_expression(Expression *exp);
Expression *create_cos_expression(Expression *exp);
Expression *create_tan_expression(Expression *exp);
Expression *create_atan_expression(Expression *exp);
Expression *create_atan_yx_expression(Expression *left, Expression *right);
Expression *create_sqr_expression(Expression *exp);
Expression *create_exp_expression(Expression *exp);
Expression *create_log_expression(Expression *exp);
Expression *create_floor_expression(Expression *exp);

Expression *identifer();

void yyerror(char *s, ...);


/*
    statment
 */
typedef struct Statement_tag Statement;

typedef enum {
    EXPRESSION_STATEMENT = 1,
    ASSIGN_EXPRESSION_STATEMENT,
    GLOBAL_STATEMENT,
    IF_STATEMENT,
    WHILE_STATEMENT,
    FOR_STATEMENT,
    NEXT_STATEMENT,
    NEXT_VAR_STATEMENT,
    PRINT_STATEMENT,
    DIM_STATEMENT,
    LABEL_STATEMENT,
    END_STATEMENT,
    GOSUB_STATEMENT,
    RETURN_STATEMENT,
    READ_STATEMENT,
    DATA_STATEMENT,
    RESTORE_STATEMENT,
    GOTO_STATEMENT,
    INPUT_STATEMENT,
    ON_GOTO_STATEMENT,
    ON_GOSUB_STATEMENT,
    COLOR_STATEMENT,
    VSYNC_STATEMENT,
    EMPTY_STATEMENT
} StatementType;

typedef struct StatementList_tag {
    Statement   *statement;
    struct StementList_tag *next;
} StatementList;

typedef struct {
    Expression  *condition;
    StatementList *statement_list;
} IfStatement;

typedef struct {
    Expression *identifier;
    Expression *expression;
} AssignStatement;

typedef struct {
    Expression  *identifier;
    Expression  *expression_init;
    Expression  *expression_to;
    Expression  *expression_step;
} ForStatement;

typedef struct {
    ExpressionList *print_array_s;
    ExpressionList *array_s;
} InputStatement;

typedef struct {
    Expression *expression_s;
    ExpressionList *array_s;
} OnGotoStatement;

typedef struct {
    Expression *expression_s;
    ExpressionList *array_s;
} OnGosubStatement;

struct Statement_tag {
    StatementType       type;
    int                 line_number;
    Boolean flag;
    union {
        char *label_s;
        Expression *expression_s;
        IfStatement if_s;
        AssignStatement assign_s;
        ExpressionList *array_s;
        ForStatement    for_s;
        InputStatement input_s;
        OnGotoStatement on_goto_s;
        OnGosubStatement on_gosub_s;

/*      WhileStatement  while_s;
        ForeachStatement        foreach_s;
        BreakStatement  break_s;
        ContinueStatement       continue_s;
        ReturnStatement return_s;
        TryStatement    try_s;
        ThrowStatement  throw_s;
 */   } u;
};

Statement *create_alloc_statement();
Statement *create_expression_statement(Expression *expression);
Statement *create_assign_statement(Expression *identifier, Expression *expression);
Statement *create_print_statement(ExpressionList *array);
Statement *create_dim_statement(ExpressionList *array);
Statement *create_if_statement(Expression *expression, StatementList *list);
Statement *create_for_statement(Expression *identifier, Expression *expression_init, Expression *expression_to, Expression *expression_step);
Statement *create_next_statement();
Statement *create_next_var_statement(Expression *var);
Statement *create_gosub_statement(char *label);
Statement *create_goto_statement(char *label);
Statement *create_restore_statement(char *label);
Statement *create_return_statement();
Statement *create_empty_statement();
StatementList *create_alloc_statement_List();
StatementList *create_statement_list(Statement *statement);
StatementList *chain_statement_list(StatementList *list, Statement *statement);
StatementList *chain_statements(StatementList *left, StatementList *right);
Statement *create_read_statement(ExpressionList *array);
Statement *create_data_statement(ExpressionList *array);
Statement *create_input_statement(ExpressionList *print_array, ExpressionList *array); 
Statement *create_on_goto_statement(Expression *expression, ExpressionList *array);
Statement *create_on_gosub_statement(Expression *expression, ExpressionList *array);
Statement *create_color_statement(Expression *expression);
Statement *create_vsync_statement(Expression *expression);

/*
    Eval
 */

#define INPUT_BUFFER_SIZE   256
#define INPUT_ARGUMENT_SIZE 256
#define VAL_BUFFER_SIZE   256

typedef enum {
    INT_BINARY_TYPE = 1,
    INT_HEX_TYPE,
    INT_TYPE,
    DOUBLE_TYPE
} NumberType;

typedef struct EvalValueList_tag EvalValueList;

typedef struct {
    IdentifierValueType type;
    int index;
    union {
        int int_value;
        double double_value;
        char *string_value;
    } u;
    
} EvalValue;


struct EvalValueList_tag {
    EvalValue value;
    struct EvalValueList_tag *next;
};


void eval_statements();
void eval_print_statement(ExpressionList *expression_list);
void eval_color_statement(Statement *statement, Expression *expression);
void eval_vsync_statement(Statement *statement, Expression *expression);
void eval_assign_expression_statement(Statement *statement);
void eval_read_statement(Statement *statement, ExpressionList *array);
void eval_data_statement(ExpressionList *array);
void eval_for_statement(Statement *statement, StatementList *pos);
int find_label_count(char *label);
StatementList *find_label(char *label);
EvalValue find_next_data(Statement *statement);
EvalValue eval_value_data(Expression *expression);
StatementList *eval_gosub_statement(Statement *statement, StatementList *pos);
StatementList *eval_goto_statement(Statement *statement, StatementList *pos);
char *get_label_list(ExpressionList *labels, int jump);
StatementList *eval_on_goto_statement(Statement *statement, StatementList *pos);
StatementList *eval_on_gosub_statement(Statement *statement, StatementList *pos);
void eval_restore_statement(Statement *statement);
int comma_count(char *buffer);
char *pointer_comma(char *buffer);
IdentifierValueType input_value_type(char *buffer);    
void eval_input_statement(Statement *statement, ExpressionList *array);
double mod(double left, double right);
EvalValue eval_expression(Expression *a);
EvalValue eval_identifier(Expression *expression);
EvalValue *search_sym_eval_value(Expression *expresson);
void print_eval(EvalValue *value);
NumberType val_value_type(char *buffer);
EvalValue eval_val_expression(Expression *expression, EvalValue *value);


/*
    Symbol
 */

typedef struct SymList_tag SymList;

typedef struct {
    char *name;
    IdentifierValueType valueType;
    IdentifierDimensionType dimensionType;
    int limit_dimension1;
    int limit_dimension2;     
    union {
        EvalValue value;
        EvalValueList *array;
    } u;
} Symbol;

struct SymList_tag {
    Symbol sym;
    struct SymList_tag *next;
};

EvalValueList *create_alloc_eval_value_list();
EvalValueList *create_init_eval_value_list(IdentifierValueType type, int index);
void chain_eval_value_list(Symbol *sym, EvalValueList *list);
void error_message_symbol();
SymList *create_alloc_symbol();
SymList *lookup(char *name, IdentifierValueType type);
void create_dim_symbol(ExpressionList *identifier_list);
void create_symbol(Expression *identifier);

/*
    String
 */

char *create_identifier(char *str);
void open_string_literial();
void add_string_literal(char ch);
char *string_value_strcat(char *s1, char *s2);
char *string_value(char *s);
void fill_string_zero(char *s, int size);
char *string_value_int(int data);
char *strmid(char *source, int location, int size);
int ascii_char(char *char_string);


/*
    Stack
 */

#define STACK_BUFFER_SIZE   512

typedef struct {
	StatementList *returnPointer;
    char *identifier;
    IdentifierValueType valueType;
    EvalValue *eval_var;
	EvalValue eval_init;
	EvalValue eval_limit;
	EvalValue eval_step;
    Boolean if_stack;
    int if_stack_count;
} ForStack;

extern ForStack forStacks[];
extern int for_stack_size;

#define GOSUB_BUFFER_SIZE 512

typedef struct {
	StatementList *returnPointer;
    Boolean if_stack;
    int if_stack_count;
} GosubStack;

extern GosubStack gosubStacks[];
extern int gosub_stack_size;

/*
    Global
 */

#define STRING_BUFFER_SIZE  512

typedef struct {
	StatementList *dataStatement;
    ExpressionList *dataExpression;
} DataPointer;

extern StatementList *top;
extern StatementList *lastPointer;

extern SymList *sym_list;
extern char *sym_string_empty;

extern int string_literial_buffer_size; 
extern char *string_literial_buffer;

extern DataPointer dataPointer;

extern ExpressionType print_type;

typedef struct {
	StatementList *executePointer;
} IFStack;

#define IF_BUFFER_SIZE  512

extern StatementList *executePointer;
extern IFStack ifStacks[];
extern int if_stack_size;
extern Boolean if_stack;
extern int if_stack_count;

extern Boolean search_next;
extern int search_next_count;

/*
    basic.c
 */

void basic_exit();
