/*

*/

%option noyywrap nodefault yylineno
%option case-insensitive

%{
	#include <stdio.h>
	#include <stdarg.h>
    #include "basic.h"
    #include "basic.tab.h"
    
    #define DEBUG   1
%}

%x COMMENTS
%x STRINGS

%%

"+"                     { return ADD; }
"-"                     { return SUB; }
"*"                     { return MUL; }
"/"                     { return DIV; }
"%"                     { return MOD; }
":"                     { return COLON; }
";"                     { return SEMICOLON; }
"("                     { return LRB; }
")"                     { return RRB; }

">"                     { return GT; }    
"<"                     { return LT; }  
">="                    { return GE; }   
"<="                    { return LE; }
"=="                    { return EQ; }
"!="                    { return NE; }   

"="                     { return EQUAL; }

"or"                    { return OR; }
"not"                   { return NOT; }
"and"                   { return AND; }
"xor"                   { return XOR; }

"dim"                   { return DIM; }

"color"                 { return COLOR; }

"end"                   { return END; }
"stop"                  { return END; }

"vsync"                  { return VSYNC; }

"read"                  { return READ; }
"data"                  { return DATA; }
"restore"               { return RESTORE; }
"if"                    { return IF; }
"then"                  { return THEN; }
"for"                   { return FOR; }
"to"                    { return TO; }
"step"                  { return STEP; }
"next"                  { return NEXT; }
"on"                    { return ON; }
"gosub"                 { return GOSUB; }
"return"                { return RETURN; }
"goto"                  { return GOTO; }

"asc"                   { return ASC; }
"chr$"                  { return CHR; }
"val"                   { return VAL; }
"str$"                  { return STR; }
"hex$"                  { return HEX; }
"mid$"                  { return MID; }
"len"                   { return LEN; }

"print"                 { return PRINT; }
"?"                     { return PRINT; }

"rnd"                   { return RND; }
"abs"                   { return ABS; }
"sgn"                   { return SGN; }
"pi"                    { return PI; }
"rad"                   { return RAD; }
"deg"                   { return DEG; }
"sin"                   { return SIN; }
"cos"                   { return COS; }
"tan"                   { return TAN; }
"atan"                  { return ATAN; }
"sqr"                   { return SQR; }
"exp"                   { return EXP; }
"log"                   { return LOG; }
"floor"                 { return FLOOR; }

"input"                 { return INPUT; }


\n                      { return NEWLINE; }

\x1a                    { printf("\\1a    \n");}

[ \t]
,                       { return COMMA;}


\"                      {
                            BEGIN(STRINGS);
                            open_string_literial();
                        }

<STRINGS>\\t            {
                            add_string_literal('\t');
                        }
<STRINGS>\\n            {
                            add_string_literal('\n');
                        }
<STRINGS>\\\"           {
                            add_string_literal('"');
                        }
<STRINGS>[^\"]          {
/*
                            printf("\"%s\"\n", yytext);
*/
                            add_string_literal(yytext[0]);
                        }
<STRINGS>\"             {
                            BEGIN(INITIAL);
                            Expression  *expression = create_alloc_expression();
                            expression->type = STRING_EXPRESSION;
                            expression->u.string_value = strdup(string_literial_buffer);
                            yylval.expression = expression;
                            return STRING_LITERIAL;
                        }


"rem"                   { 
/*
                            printf("COMMENT BEGIN\n");
*/
                            BEGIN(COMMENTS);
                        }

\'                      { 
/*
                          printf("COMMENT BEGIN\n");
*/
                          BEGIN(COMMENTS);
                        }

<COMMENTS>[^\n]*        { 
/*
                            printf("\"%s\"\n", yytext); 
*/
                        }

<COMMENTS>\n            { 
/*
                            printf("COMMENT END\n");
*/
                            BEGIN(INITIAL);
                        }


[a-z_]+[a-z0-9_]*       {
/*                            printf("var    = %s\n", yytext);*/
                            yylval.identifier = create_identifier(yytext);
                            return IDENTIFIER;
                        }

[a-z_]+[a-z0-9_]*\$     {
/*
                            printf("var$    = %s\n", yytext);
*/
                            yylval.identifier = create_identifier(yytext);

                            return IDENTIFIER_STRING;
                        }

@[a-z_]+[a-z0-9_]*      {
/*
                            printf("var$    = %s\n", yytext);
*/
                            yylval.identifier = create_identifier(yytext);

                            return LABEL;
                        }

&h[0-9a-f]+             {
                            Expression  *expression = create_alloc_expression();
                            expression->type = INT_EXPRESSION;
                            sscanf(&yytext[2], "%x", &expression->u.int_value);
                            yylval.expression = expression;
                            return INT_LITERAL;
                        }

&b[01]+                 {
                            size_t size;
                            int i;
                            int value = 0;
                            size = strlen(yytext);
                            
                            if (yytext[2] == '1') {
                                    value = value + 1;
                            }
                            for (i=3;i<size;i++) {
                                value = value * 2;
                                if (yytext[i] == '1') {
                                    value = value + 1;
                                }                               
                            }
                            Expression  *expression = create_alloc_expression();
                            expression->type = INT_EXPRESSION;
                            expression->u.int_value = value;
                            yylval.expression = expression;
                            return INT_LITERAL;
                        }

"0"                     {
                            Expression  *expression = create_alloc_expression();
                            expression->type = INT_EXPRESSION;
                            expression->u.int_value = 0;
                            yylval.expression = expression;
                            return INT_LITERAL;
                        }

[1-9][0-9]*             {
                            Expression  *expression = create_alloc_expression();
                            expression->type = INT_EXPRESSION;
                            sscanf(yytext, "%d", &expression->u.int_value);
                            yylval.expression = expression;
                            return INT_LITERAL;
                        }

[0-9]+\.[0-9]+          {
                            Expression  *expression = create_alloc_expression();
                            expression->type = DOUBLE_EXPRESSION;
                            sscanf(yytext, "%lf", &expression->u.double_value);
                            yylval.expression = expression;
                            return DOUBLE_LITERAL;
                        }

.                       {
                            printf("\x1b[31m未定義の文字 '%c' が見つかりました。\x1b[30m\n", *yytext);
                            exit(1);
                        }

%%
